package sim.app.ubik.domoticDevices;

import java.io.IOException;
import java.io.StringReader;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import ocp.service.ContextEntityItems;
import ocp.service.ContextItemFloat;
import ocp.service.ContextService;
import org.apache.log4j.Logger;
import sim.app.ubik.Ubik;
import sim.app.ubik.people.Person;
import sim.app.ubik.people.Person.WeightItem;
import sim.engine.SimState;
import sim.util.Int2D;
import sim.util.MutableInt2D;
import ubik3d.model.HomePieceOfFurniture;

/**
 *
 * @author Chema
 */
public class DistanceSensor extends FixedDomoticDevice {

	private int reach = -1;
	private float alpha = Float.NaN;
	private float distanceDetected = Integer.MAX_VALUE;

	public DistanceSensor(int floor, HomePieceOfFurniture device3dModel, Ubik ubik) {
		super(floor, device3dModel, ubik);
		ocp = ubik.createOCPProxyProducer("UBIK.DistanceSensor-" + this.getId());
		try {
			Properties properties = new Properties();
			StringReader sr = new StringReader(device3dModel.getMetadata());
			properties.load(sr);
			String reachStr = properties.getProperty("reach");
			String alphaStr = properties.getProperty("alpha");
			if (reachStr != null)
				reach = Integer.parseInt(reachStr);
			if (alphaStr != null)
				alpha = Float.parseFloat(alphaStr);
		} catch (IOException e) {
			Logger.getLogger(this.getClass()).error(e.getLocalizedMessage(), e);
		}
	}

	private boolean withinCone(Int2D p) {
		MutableInt2D pos = this.getPosition();
		return Math.abs(Math.atan2(p.getY() - pos.getY(), p.getX() - pos.getX()) + Math.PI - this.orientation2D())
				< alpha / 2.0;
	}

	@Override
	public void step(SimState state) {
		float distance = Integer.MAX_VALUE;
		List people = ubik.getBuilding().getFloor(this.getFloor()).getPersonHandler().getPersons(
				this.getPosition().getX(), this.getPosition().getY(), reach / this.cellSize);
		for (Iterator iterator = people.iterator(); iterator.hasNext();) {
			Person p = (Person) iterator.next();
			for (Iterator iterator1 = p.getWeightModel().iterator(); iterator1.hasNext();) {
				WeightItem item = (WeightItem) iterator1.next();
				if (withinCone(item.getPosition()))
					distance = (float) this.getPosition().distance(item.getPosition()) * this.cellSize;
			}

		}

		if (distance != distanceDetected && ocp != null) {
			ContextEntityItems cei = new ContextEntityItems(getClass().getSimpleName(), getId());
			cei.addContextItem(new ContextItemFloat("distanceDetected", distance));
			ocp.getContextService().setContextItems(cei);
		}
		distanceDetected = distance;
	}

	@Override
	public void stop() {
	}

	@Override
	public void createInOCP(ContextService cs) {
		super.createInOCP(cs);
		ContextEntityItems cei = new ContextEntityItems(getClass().getSimpleName(), getId());
		cei.addContextItem(new ContextItemFloat("width", device3DModel.getWidth()));
		cei.addContextItem(new ContextItemFloat("depth", device3DModel.getDepth()));
		cei.addContextItem(new ContextItemFloat("reach", reach));
		cei.addContextItem(new ContextItemFloat("angle", alpha));
		cei.addContextItem(new ContextItemFloat("distanceDetected", distanceDetected));
		cs.setContextItems(cei);
	}
}
